#<profile4xi profile = "02" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "pipelistener4xi.py">
#<abstract4xi abstract = "provides the class PipeListener4Xi"/>
#<refer4xi reference = "connection.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------


# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'PipeListener4Xi' ] 
#"
#/>


# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import errno
import io
import itertools
import os
import pickle
import select
import socket
import struct
import sys
import time
import tempfile
try:
    import _winapi
    from _winapi import WAIT_OBJECT_0, WAIT_TIMEOUT, INFINITE
except ImportError:
    if sys.platform == 'win32':
        raise
    _winapi = None 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
from multiprocessing import connection
try:                             # Python ones
    from multiprocessing.connection import PipeListener
except:
    print("pipelistener4xi.py *** failed in from multiprocessing.connection import PipeListener ***") 
try:                             
    from multiprocessing.connection import PipeConnection
except:
    print("pipelistener4xi.py *** failed in from multiprocessing.connection import PipeConnection ***")     
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi

from constant_family4xi import ConstantFamily4Xi
from constant_connection4xi import ConstantConnection4Xi 
from constant_system4xi import ConstantSystem4Xi

from examine4xi import examine4xi

from finalize4xi import Finalize4Xi

from loop4xi import limit2loop4xi

# from pipeconnection4xi import PipeConnection4Xi
from print4xi import print4xi

from reason_object4xi import reason_object4xi

from sense_object4xi import sense_object4xi
from sleep4xi import omega2sleep4xi

from time4xi import time4xi
#/>

# -------------------------------------------------------------------------------------------------
# Settings
# -------------------------------------------------------------------------------------------------
#<set4xi setting = "
cs2xi = ConstantSystem4Xi()
#"/>


if sys.platform == cs2xi.get_windows():
    # =============================================================================================
    #<class4xi class = "PipeListener4Xi" base = "object" child = "None" synopsis = "PipeListener4Xi class represents a named pipe" usage = "
    #   connection.py
    #       Listener
    #           if family == 'AF_PIPE':
    #               self._listener = PipeListener(address, backlog)
    #
    # listener4xi.py:
    #     class Listener4Xi(object):
    #         self._listener = PipeListener4Xi(address2xi, backlog) 
    #"
    #>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # =============================================================================================
    class PipeListener4Xi(object):
        '''
        PipeListener4Xi class represents a named pipe
        '''
        #                                        
        # ---------------------------------------------------------------------------------------------
        # sharable class variables
        # ---------------------------------------------------------------------------------------------      
        #<share4xi share = " 
        _cc2xi = ConstantConnection4Xi()
        _buffersize2xi = _cc2xi.get_buffersize()        
        _cf2xi = ConstantFamily4Xi()
        #"
        #/>
        
        # -----------------------------------------------------------------------------------------
        #<methodize4xi method = "init-of-PipeListener4Xi" usage = "initializes PipeListener4Xi">
        #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
        # -----------------------------------------------------------------------------------------
        def __init__(self, mountain, backlog=None):
            '''
            initializes PipeListener4Xi with (self, mountain, backlog) 
            '''
            
            # ---------------------------------------------------------------------------------------------
            # Public API of this class
            # ---------------------------------------------------------------------------------------------    
            #<expose4xi expose = "
            _exposed_ = ('accept', 'sense', 'reason', 'ace')
            #" />

            # -----------------------------------------------------------------------------------------    
            # (Step 1/4)
            # -----------------------------------------------------------------------------------------
            class2print = "PipeListener4Xi"
            method2print = "__init__"

            message2print = "{(Step 1/4a) started" + "; init-of-PipeListener4Xi-started: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)   
            # 08:50:41 03/14/13 Pacific Daylight Time
            
            message2print = "{(Step 1/4b) borning" + "; pipelistener2xi-of-PipeListener4Xi-borning: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 08:50:41 03/14/13 Pacific Daylight Time
            
            message2print = "{(Step 1/4c) inheriting" + "; object-inheriting-by-pipelistener2xi-of-PipeListener4Xi: " + str(object)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            # <class 'object'>
                            
            message2print = "{(Step 1/4d) passed" + "; mountain-passed-to-init-of-PipeListener4Xi: " + str(mountain) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                       
            # \\.\pipe\pyc-3824-0-fi0jby
            
            message2print = "{(Step 1/4e) passed" + "; backlog-passed-to-init-of-PipeListener4Xi: " + str(backlog) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                       
            # 1
            
            if examine4xi(class2print, method2print):
                # -----------------------------------------------------------------------------------------
                # (StepOriginal):        
                # ----------------------------------------------------------------------------------------- 
                PipeListener.__init__(self, mountain, backlog)
                
                message2print = "{(StepOriginal) performed" + "; Original-PipeListener-performed-by-init-of-Listener4Xi: " + str(PipeListener.__init__) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #        
            else:        
                    
            
                # -----------------------------------------------------------------------------------------    
                # (Step 2/4)
                # -----------------------------------------------------------------------------------------
                if mountain is None:
                    family2xi = PipeListener4Xi._cf2xi.get_default_family()          #dragonized by forcing to the right family! 
                
                    message2print = "{(Step 2/4a) set" + "; family2xi-set-by-init-of-Listener4Xi to: " + str(family2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    

                    mountain = arbitrary_address4xi(family2xi)                       #dragonized by forcing an mountain
                
                    message2print = "{(Step 2/4b) set" + "; mountain-of-listener4xi-set-by-init-of-Listener4Xi to: " + str(mountain)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                else:
                    pass

                    self._address = mountain                                         #the mountain is given to the caller object listener2xi
                
                message2print = "{(Step 2/4c) set" + "; mountain-of-listener2xi-set-by-init-of-PipeListener4Xi to: " + str(self._address)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                # \\.\pipe\pyc-3824-0-fi0jby
                    
                self._handle_queue = [self._new_handle(first=True)]
                
                message2print = "{(Step 2/4d) set" + "; queue-of-listener2xi-set-by-init-of-PipeListener4Xi to: " + str(self._handle_queue)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # [252] 
                    
                self._last_accepted = None
                
                message2print = "{(Step 2/4e) set" + "; last_accepted-of-listener2xi-set-by-init-of-PipeListener4Xi to: " + str(self._last_accepted)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # None              
                
                # -----------------------------------------------------------------------------------------    
                # (Step 3/4)
                # -----------------------------------------------------------------------------------------    
                message2print = "{(Step 3/4a) passing" + "; queue-passing-to-Finalize4Xi-by-init-of-PipeListener4Xi: " + str(self._handle_queue) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # [252]
                
                message2print = "{(Step 3/4b) passing" + "; mountain-passing-to-Finalize4Xi-by-init-of-PipeListener4Xi: " + str(self._address) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # \\.\pipe\pyc-3824-0-fi0jby
                
                self.close = Finalize4Xi(
                    self, PipeListener4Xi._finalize_pipe_listener,
                    args=(self._handle_queue, self._address), exitpriority=0
                    )
               
                message2print = "{(Step 3/4c) set" + "; close-of-listener2xi-set-by-init-of-PipeListener4Xi to: " + str(self.close)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)      
                # <finalize4xi: __repr__: Finalize object, callback=_finalize_pipe_listener, args=([252], '\\\\.\\pipe\\pyc-3824-0-fi0jby'), exitprority=0>

        
            # -----------------------------------------------------------------------------------------    
            # (Step 4/4)
            # -----------------------------------------------------------------------------------------    
            message2print = "{(Step 4/4b) activated" + "; pipelistener2xi-of-PipeListener4Xi-activated: " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 08:50:41 03/14/13 Pacific Daylight Time    
                   
            message2print = "{(Step 4/4c) returning" + "; init-of-PipeListener4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 08:50:41 03/14/13 Pacific Daylight Time
                            
            return              

        #</methodize4xi>
        
        
        # -----------------------------------------------------------------------------------------
        #<methodize4xi method = "_new_handle-of-PipeListener4Xi" usage = "new handle with (self, first=False)">
        #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
        # -----------------------------------------------------------------------------------------
        def _new_handle(self, first=False): 
            '''
            new handle with (self, first=False)
            '''

            # -----------------------------------------------------------------------------------------    
            # (Step 1/4)
            # -----------------------------------------------------------------------------------------
            class2print = "PipeListener4Xi"
            method2print = "_new_handle"

            message2print = "{(Step 1/4a) started" + "; _new_handle-of-PipeListener4Xi-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 08:50:41 03/14/13 Pacific Daylight Time
            
            message2print = "{(Step 1/4b) passed" + "; first-passed-to-_new_handle-of-PipeListener4Xi: " + str(first) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                       
            # True
            
            if examine4xi(class2print, method2print):
                # -----------------------------------------------------------------------------------------
                # (StepOriginal):        
                # ----------------------------------------------------------------------------------------- 
                message2print = "{(StepOriginal) performing" + "; Original-PipeListener-performed-by-init-of-Listener4Xi: " + str(PipeListener.__init__) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #       
                return PipeListener._new_handle(self, first)
                
 
            else:      
            
                # -----------------------------------------------------------------------------------------    
                #     (Step 2/4)
                # -----------------------------------------------------------------------------------------
                flags2xi = _winapi.PIPE_ACCESS_DUPLEX | _winapi.FILE_FLAG_OVERLAPPED
                
                message2print = "{(Step 2/4a) set" + "; flags2xi-set-by-_new_handle-of-PipeListener4Xi to: " + str(flags2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                # 1073741827
                
                # -----------------------------------------------------------------------------------------    
                # (Step 3/4)
                # -----------------------------------------------------------------------------------------    
                if first:
                    flags2xi |= _winapi.FILE_FLAG_FIRST_PIPE_INSTANCE
                    
                    message2print = "{(Step 3/4a) set" + "; flags2xi-set-by-_new_handle-of-PipeListener4Xi to: " + str(flags2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 1074266115
                    
                # -----------------------------------------------------------------------------------------    
                # (Step 4/4)
                # -----------------------------------------------------------------------------------------    
                message2print = "{(Step 4/4a) calling" + "; _winapi.CreateNamedPipe-calling-by-_new_handle-of-PipeListener4Xi to: " + str(_winapi.CreateNamedPipe)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <built-in function CreateNamedPipe>
                
                message2print = "{(Step 4/4b) returning" + "; _new_handle-of-PipeListener4Xi-returning: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print) 
                # 08:50:41 03/14/13 Pacific Daylight Time
                
                return _winapi.CreateNamedPipe(
                    self._address, flags2xi,
                    _winapi.PIPE_TYPE_MESSAGE | _winapi.PIPE_READMODE_MESSAGE |
                    _winapi.PIPE_WAIT,
                    _winapi.PIPE_UNLIMITED_INSTANCES, PipeListener4Xi._buffersize2xi, PipeListener4Xi._buffersize2xi, 
                    _winapi.NMPWAIT_WAIT_FOREVER, _winapi.NULL
                    )

        #</methodize4xi>
                             
        # -----------------------------------------------------------------------------------------
        #<methodize4xi method = "accept-of-PipeListener4Xi" usage = "accepts with (self)">
        #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
        # -----------------------------------------------------------------------------------------
        def accept(self):
            '''
            PipeListener4Xi method accept: accepts with (self) 
            '''
            # -----------------------------------------------------------------------------------------    
            # (Step 1/5)
            # -----------------------------------------------------------------------------------------    
            class2print = "PipeListener4Xi"
            method2print = "accept"

            message2print = "{(Step 1/5a) started" + "; accept-of-PipeListener4Xi-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 08:50:41 03/14/13 Pacific Daylight Time
  
            if examine4xi(class2print, method2print):
                # -----------------------------------------------------------------------------------------
                # (StepOriginal):        
                # ----------------------------------------------------------------------------------------- 
                message2print = "{(StepOriginal) performing" + "; Original-PipeListener-performed-by-init-of-Listener4Xi: " + str(PipeListener.accept) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #       
                return PipeListener.accept(self)
            
            else:      
            
                # -----------------------------------------------------------------------------------------    
                # (Step 2/5)
                # -----------------------------------------------------------------------------------------    
                newhandle2xi = self._new_handle()
                
                message2print = "{(Step 2/5a) passing" + "; newhandle2xi-passing-to-handle-queue-append-of-pipelistener2xi-by-accept-of-PipeListener4Xi: " + str(newhandle2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                # 288; 292
                    
                self._handle_queue.append(newhandle2xi)
                
                message2print = "{(Step 2/5b) added" + "; newhandle2xi-added-to-handle-queue-append-of-pipelistener2xi-by-accept-of-PipeListener4Xi: " + str(newhandle2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)              
                # 288; 292
                            
                # -----------------------------------------------------------------------------------------    
                # (Step 3/5)
                # -----------------------------------------------------------------------------------------  
                handle2xi = self._handle_queue.pop(0)
                
                message2print = "{(Step 3/5a) got" + "; handle2xi-got-by-accept-of-PipeListener4Xi: " + str(handle2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 260; 288
                
                # -----------------------------------------------------------------------------------------    
                # (Step 4/5) - JATKA EHDOTTOMASTI TASTA - MIKSI KUTSUTAAN USEIN ? MIKSI res2xi None TOISELLA KERRALLA ? 
                # -----------------------------------------------------------------------------------------    
                try:
                    ov2xi = _winapi.ConnectNamedPipe(handle2xi, overlapped=True)
                    
                    message2print = "{(Step 4/5a) got" + "; ov2xi-got-by-accept-of-PipeListener4Xi: " + str(ov2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # <_winapi.Overlapped object at 0x000000000367EAE0>
                    
                except OSError as e:
                    if e.winerror != _winapi.ERROR_NO_DATA:
                        message2print = "{(Step 4/5z1) excepted" + "; winerror-excepted-by-accept-of-PipeListener4Xi: " + str(e.winerror) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        raise
                        # ERROR_NO_DATA can occur if a client has already connected,
                        # written data and then disconnected -- see Issue 14725.
                        
                    # comes here if there is no exception above!    
                else:
                    res2xi = None
                    try:
                        message2print = "{(Step 4/5b) calling" + "; _winapi.WaitForMultipleObjects-calling-by-accept-of-PipeListener4Xi: " + str(_winapi.WaitForMultipleObjects) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # <built-in function WaitForMultipleObjects> 2x!
                        
                        res2xi = _winapi.WaitForMultipleObjects(
                            [ov2xi.event], False, INFINITE)
                            
                        message2print = "{(Step 4/5c) got" + "; res2xi-got-by-accept-of-PipeListener4Xi: " + str(res2xi) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # <_winapi.Overlapped object at 0x000000000367ED40>
                            
                    except:
                        message2print = "{(Step 4/5d) calling" + "; cancel-calling-by-accept-of-PipeListener4Xi: " + str(ov2xi.cancel) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # <built-in method cancel of _winapi.Overlapped object at 0x000000000367EAE0>
                        
                        ov2xi.cancel()
                        
                        message2print = "{(Step 4/5e) calling" + "; _winapi.CloseHandle-calling-by-accept-of-PipeListener4Xi: " + str(_winapi.CloseHandle) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # <built-in function CloseHandle>
                        
                        _winapi.CloseHandle(handle2xi)
                        
                        
                        message2print = "{(Step 4/5z2) failed" + "; res2xi-failed-by-accept-of-PipeListener4Xi: " + str(res2xi) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # None
                        
                        raise
                    finally:
                        message2print = "{(Step 4/5f) calling" + "; GetOverlappedResult-calling-by-accept-of-PipeListener4Xi: " + str(ov2xi.GetOverlappedResult) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        
                        _, err = ov2xi.GetOverlappedResult(True)
                        
                        if err == 0:
                            message2print = "{(Step 4/5g) performed" + "; GetOverlappedResult-performed-by-accept-of-PipeListener4Xi: " + str(ov2xi.GetOverlappedResult) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                        else:
                            message2print = "{(Step 4/5z3) failed" + "; err-failed-by-accept-of-PipeListener4Xi: " + str(err) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)   
    
                # -----------------------------------------------------------------------------------------    
                # (Step 5/5)
                # -----------------------------------------------------------------------------------------    
                message2print = "{(Step 5/5a) passing" + "; handle2xi-passing-to-PipeConnection4Xi-by-accept-of-PipeListener4Xi: " + str(handle2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                    
                message2print = "{(Step 5/5b) returning" + "; PipeConnection4Xi-function-returning-by-accept-of-PipeListener4Xi: " + str(PipeConnection4Xi) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)  
                                                       
                      
                return PipeConnection4Xi(handle2xi) # JATKA why was return PipeConnection(handle2xi) ??
                
        #</methodize4xi>
            
        @staticmethod
        # -----------------------------------------------------------------------------------------
        #<methodize4xi method = "finalize-pipe-listener-of-PipeListener4Xi" usage = "finalizes with (queue, mountain)">
        #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
        # -----------------------------------------------------------------------------------------
        def _finalize_pipe_listener(queue, mountain):
            '''
            finalizes with (queue, mountain)
            '''
   
            # -------------------------------------------------------------------------------------    
            # (Step 1/3)
            # -------------------------------------------------------------------------------------     
                
            class2print = "PipeListener4Xi"
            method2print = "_finalize_pipe_listener"

            message2print = "{(Step 1/3a) started" + "; finalize-pipe-listener-of-PipeListener4Xi-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
    
            message2print = "{(Step 1/3b) passed" + "; queue-passed-to-finalize-pipe-listener-of-PipeListener4Xi: " + str(queue) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                     
    
            message2print = "{(Step 1/3c) passed" + "; mountain-passed-to-finalize-pipe-listener-of-PipeListener4Xi: " + str(mountain) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                     
               
            if mountain is None:
                message2print = "{(Step 1/3d) warned" + "; mountain-by-finalize-pipe-listener-of-PipeListener4Xi: " + " ** " + str(mountain)  + " ** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)               
            
            if examine4xi(class2print, method2print):
                # -----------------------------------------------------------------------------------------
                # (StepOriginal):        
                # ----------------------------------------------------------------------------------------- 
                message2print = "{(StepOriginal) performing" + "; Original-PipeListener-performed-by-init-of-Listener4Xi: " + str(PipeListener._finalize_pipe_listener) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #       
                return PipeListener._finalize_pipe_listener(queue, mountain)
            
            else:      
              
                # -----------------------------------------------------------------------------------------    
                # (Step 2/3)
                # -----------------------------------------------------------------------------------------    
                for handle2xi in queue:
                    
                    message2print = "{(Step 2/3a) passing" + "; handle2xi-passing-to-close4xi-by-finalize-pipe-listener-of-PipeListener4Xi: " + str(handle2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                    
                 
                    close4xi(handle2xi)       # dragonized so that system tries several close functions, if necessary  
    
                    message2print = "{(Step 2/3b) performed" + "; close-performed-by-finalize-pipe-listener-of-PipeListener4Xi: " + str(close) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                                                  
                    
                # -----------------------------------------------------------------------------------------    
                # (Step 3/3)
                # -----------------------------------------------------------------------------------------    
                message2print = "{(Step 3/3) returning" + "; finalize-pipe-listener-of-PipeListener4Xi-returning: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print) 
                
                return            

        #</methodize4xi>
                                           
        # ---------------------------------------------------------------------------------------------
        #<methodize4xi method = "sense-of-PipeListener4Xi" usage = "senses PipeListener4Xi class objects">
        #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
        # ---------------------------------------------------------------------------------------------
        def sense(self):
            '''
            senses PipeListener4Xi class objects
            '''
            
            
            class2print = 'PipeListener4Xi'
            method2print = 'sense'
            

            message2print = "{started" + "; sense-of-PipeListener4Xi-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
    
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------  
            message2print = "{sensing" + "; PipeListener4Xi-sensing-by-sense-of-PipeListener4Xi: " + str(PipeListener4Xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
                       
            
            # -----------------------------------------------------------------------------------------
            # get sensed knowledge for this object
            # -----------------------------------------------------------------------------------------        
            kb2xi = sense_object4xi(self)
            
            message2print = "{got" + "; kb2xi-got-by-sense-of-PipeListener4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
            
            # -----------------------------------------------------------------------------------------
            # add knowledge specific to this object
            # -----------------------------------------------------------------------------------------         
            try:
                kb2xi['address-of-pipelistener2xi'] = self._address          
                kb2xi['handle-queue-of-pipelistener2xi'] = self._handle_queue             
                kb2xi['last-accepted-of-pipelistener2xi'] = self._last_accepted 
                kb2xi['close-of-pipelistener2xi'] = self.close           
            except:
                message2print = "{yielded" + "; kb2xi-yielded-by-sense-of-PipeListener4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
            
                                              
            message2print = "{sensed" + "; PipeListener4Xi-sensed-by-sense-of-PipeListener4Xi: " + str(PipeListener4Xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            
            # -----------------------------------------------------------------------------------------
            # 
            # -----------------------------------------------------------------------------------------         
            message2print = "{returning" + "; kb2xi-returning-by-sense-of-PipeListener4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                                  
              
            return kb2xi
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------        
        #</methodize4xi>
        
            
        # ---------------------------------------------------------------------------------------------
        #<methodize4xi method = "reason-of-K4tagging; NACK4modul
        #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
        # ---------------------------------------------------------------------------------------------
        def reason(self):
            '''
            method reason: reasons PipeListener4Xi class objects
            '''
                  
            
            class2print = 'PipeListener4Xi'
            method2print = 'reason'
    

            message2print = "{started" + "; reason-of-PipeListener4Xi-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
        
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------
            message2print = "{reasoning" + "; PipeListener4Xi-reasoning-by-reason-of-PipeListener4Xi: " + str(PipeListener4Xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                
            
            # -----------------------------------------------------------------------------------------
            # get reasoned knowledge for this object
            # -----------------------------------------------------------------------------------------         
            kb4xi = reason_object4xi(self)
            
            
            message2print = "{got" + "; kb4xi-got-by-reason-of-PipeListener4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
            
            # -----------------------------------------------------------------------------------------
            # add knowledge specific to this object
            # -----------------------------------------------------------------------------------------          

              
            
            message2print = "{reasoned" + "; PipeListener4Xi-reasoned-by-reason-of-PipeListener4Xi: " + str(PipeListener4Xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------        
            message2print = "{returning" + "; kb4xi-returning-by-reason-of-PipeListener4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                              
              
            return kb4xi
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------            
        #</methodize4xi>
        
        
        # ---------------------------------------------------------------------------------------------
        #<methodize4xi method = "ace-of-PipeListener4Xi" usage = "aces PipeListener4Xi class objects">
        #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
        # ---------------------------------------------------------------------------------------------
        def ace(self):
            '''
            method ace: aces PipeListener4Xi class objects 
            '''
   
            
            class2print = 'PipeListener4Xi'
            method2print = 'ace'
            
            message2print = "{started" + "; ace-of-PipeListener4Xi-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
       
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------
            message2print = "{acing" + "; PipeListener4Xi-acing-by-ace-of-PipeListener4Xi: " + str(PipeListener4Xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
                     
            
            # -----------------------------------------------------------------------------------------
            # get aced knowledge for this object
            # -----------------------------------------------------------------------------------------        
            kb8xi = ace_object4xi(self)  
            
            message2print = "{got" + "; kb8xi-got-by-ace-of-PipeListener4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
    
    
            # -----------------------------------------------------------------------------------------
            # add knowledge specific to this object
            # -----------------------------------------------------------------------------------------              
            
              
            message2print = "{aced" + "; PipeListener4Xi-aced-by-ace-of-PipeListener4Xi: " + str(PipeListener4Xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                   
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------        
            message2print = "{returning" + "; kb8xi-returning-by-ace-of-PipeListener4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                                  
            
            return kb8xi
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------                                 
        #</methodize4xi>
    #</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of pipelistener4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>



